<?php
/**
 * @file
 * Page callbacks for Twitter module.
 */

/**
 * Twitter settings form.
 */
function twitter_admin_form() {
  $form['twitter_import'] = array(
    '#type' => 'checkbox',
    '#title' => t('Import and display the Twitter statuses of site users who have entered their Twitter account information.'),
    '#default_value' => variable_get('twitter_import', 1),
  );
  $form['twitter_expire'] = array(
    '#type' => 'select',
    '#title' => t('Delete old statuses'),
    '#default_value' => variable_get('twitter_expire', 0),
    '#options' => array(0 => t('Never')) + drupal_map_assoc(array(604800, 2592000, 7776000, 31536000), 'format_interval'),
  );

  $form['oauth'] = array(
    '#type' => 'fieldset',
    '#title' => t('OAuth Settings'),
    '#access' => module_exists('oauth_common'),
    '#description' => t('To enable OAuth based access for twitter, you must <a href="@url">register your application</a> with Twitter and add the provided keys here.', array('@url' => 'https://dev.twitter.com/apps/new')),
  );
  $form['oauth']['callback_url'] = array(
    '#type' => 'item',
    '#title' => t('Callback URL'),
    '#value' => url('twitter/oauth', array('absolute' => TRUE)),
  );
  $form['oauth']['twitter_consumer_key'] = array(
    '#type' => 'textfield',
    '#title' => t('OAuth Consumer key'),
    '#default_value' => variable_get('twitter_consumer_key', NULL),
  );
  $form['oauth']['twitter_consumer_secret'] = array(
    '#type' => 'textfield',
    '#title' => t('OAuth Consumer secret'),
    '#default_value' => variable_get('twitter_consumer_secret', NULL),
  );

  // Twitter external APIs settings.
  $form['twitter'] = array(
    '#type' => 'fieldset',
    '#title' => t('Twitter Settings'),
    '#description' => t('The following settings connect Twitter module with external APIs. ' .
      'Change them if, for example, you want to use Identi.ca.'),
  );
  $form['twitter']['twitter_host'] = array(
    '#type' => 'textfield',
    '#title' => t('Twitter host'),
    '#default_value' => variable_get('twitter_host', TWITTER_HOST),
  );
  $form['twitter']['twitter_api'] = array(
    '#type' => 'textfield',
    '#title' => t('Twitter API'),
    '#default_value' => variable_get('twitter_api', TWITTER_API),
  );
  $form['twitter']['twitter_search'] = array(
    '#type' => 'textfield',
    '#title' => t('Twitter search'),
    '#default_value' => variable_get('twitter_search', TWITTER_SEARCH),
  );
  $form['twitter']['twitter_tinyurl'] = array(
    '#type' => 'textfield',
    '#title' => t('TinyURL'),
    '#default_value' => variable_get('twitter_tinyurl', TWITTER_TINYURL),
  );

  return system_settings_form($form);
}

/**
 * Form builder that lists Twitter accounts.
 */
function twitter_user_settings() {
  // Verify OAuth keys.
  if (!twitter_api_keys()) {
    $output = t('<p>You need to authenticate at least one Twitter account in order to use the Twitter API. Please fill out the OAuth fields at <a href="/admin/settings/twitter/settings">Twitter Settings</a> and then return here.</p>');
  }
  else {
    module_load_include('inc', 'twitter');
    $twitter_accounts = twitter_account_load_all();

    $output = array();
    if (count($twitter_accounts)) {
      // List Twitter accounts.
      $output = t('<p>Tweets are pulled from Twitter by ' .
            '<a href="/admin/reports/status/run-cron?destination=admin/settings/twitter">running cron</a>. ' .
            'You can view the full list of tweets at the <a href="/tweets">Tweets</a> view.</p>');
      $output .= drupal_get_form('twitter_account_list_form', $twitter_accounts);
    }
    else {
      // No accounts added. Inform about how to add one.
      $output = t('<p>No Twitter accounts have been added yet. Click on the following button to add one.</p>');
    }

    $add_account = array(
      '#type' => 'fieldset',
      '#title' => t('Add Twitter accounts'),
      '#weight' => 5,
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#children' => '',
    );

    if (user_access('add authenticated twitter accounts')) {
      $add_account['#children'] .= drupal_get_form('twitter_auth_account_form');
    }
    if (twitter_connect()) {
      $add_account['#children'] .= drupal_get_form('twitter_non_auth_account_form');
    }
    $output .= drupal_render($add_account);
  }

  return $output;
}

/**
 * Formats each Twitter account as a row within a form.
 */
function twitter_account_list_form($form_state, $twitter_accounts = array()) {
  $form['#tree'] = TRUE;
  $form['accounts'] = array();

  foreach ($twitter_accounts as $twitter_account) {
    $form['accounts'][] = _twitter_account_list_row($twitter_account);
  }

  if (!empty($twitter_accounts)) {
    $form['buttons']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save changes'),
    );
  }

  return $form;
}

/**
 * Returns the form fields to manage a Twitter account.
 */
function _twitter_account_list_row($account) {
  $form['#account'] = $account;

  $form['id'] = array(
    '#type' => 'value',
    '#value' => $account->id,
  );

  $form['screen_name'] = array(
    '#type' => 'value',
    '#value' => $account->screen_name,
  );

  $form['image'] = array(
    '#value' => theme('image', $account->profile_image_url, '', '', array(), FALSE),
  );

  $form['visible_name'] = array(
    '#value' => l($account->screen_name, 'http://www.twitter.com/' . $account->screen_name),
  );

  $form['description'] = array(
    '#value' => filter_xss($account->description),
  );

  $form['auth'] = array(
    '#value' => $account->is_auth() ? t('Yes') : t('No'),
  );

  $form['protected'] = array(
    '#value' => empty($account->protected) ? t('No') : t('Yes'),
  );

  $form['import'] = array(
    '#type' => 'checkbox',
    '#default_value' => $account->import ? $account->import : '',
  );
  if ($account->import == TRUE) {
    $form['import']['#suffix'] = l('View', 'tweets/' . $account->screen_name, array('attributes' => array('target' => '_blank')));
  }

  if ($account->is_auth()) {
    $form['mentions'] = array(
      '#type' => 'checkbox',
      '#default_value' => $account->mentions ? $account->mentions : '',
    );
  }
  else {
    $form['mentions'] = array(
      '#value' => '',
    );
  }

  $form['delete'] = array(
    '#type' => 'checkbox',
  );

  return $form;
}

/**
 * Themes the list of Twitter accounts.
 */
function theme_twitter_account_list_form($form) {
  $header = array(
    '',
    t('Name'),
    t('Description'),
    t('Auth'),
    t('Private'),
    t('Tweets'),
    t('Mentions'),
    t('Delete'),
  );

  $rows = array();
  foreach (element_children($form['accounts']) as $key) {
    $element = &$form['accounts'][$key];
    $row = array(
      drupal_render($element['image']),
      drupal_render($element['id']) . drupal_render($element['screen_name']) .
        drupal_render($element['visible_name']),
      drupal_render($element['description']),
      drupal_render($element['auth']),
      drupal_render($element['protected']),
      drupal_render($element['import']),
      drupal_render($element['mentions']),
      drupal_render($element['delete']),
    );
    $rows[] = $row;
  }

  $output = theme('table', $header, $rows);
  $output .= drupal_render($form);
  return $output;
}

/**
 * Form submit handler for altering the list of Twitter accounts.
 */
function twitter_account_list_form_submit($form, &$form_state) {
  $accounts = $form_state['values']['accounts'];
  foreach ($accounts as $account) {
    if (empty($account['delete'])) {
      twitter_account_save($account);
    }
    else {
      $twitter_account = twitter_account_load($account['id']);
      twitter_account_delete($account['id']);
      drupal_set_message(t('The Twitter account <em>!account</em> was deleted.',
        array('!account' => $twitter_account->screen_name)));
    }
  }
  drupal_set_message(t('The Twitter account settings were updated.'));
}

/**
 * Form to add an authenticated Twitter account.
 */
function twitter_auth_account_form(&$form_state) {
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Go to Twitter to add an authenticated account'),
    '#prefix' => t('Authenticated accounts can post, sign in and pull mentions. ' .
                   'At least one authenticated account is needed for Twitter ' .
                   'module to work.</br>'),
  );

  return $form;
}

/**
 * Form validation for adding a new Twitter account.
 */
function twitter_auth_account_form_validate($form, &$form_state) {
  $key = variable_get('twitter_consumer_key', '');
  $secret = variable_get('twitter_consumer_secret', '');
  if ($key == '' || $secret == '') {
    form_set_error('', t('Please configure your consumer key and secret key at ' .
      '<a href="!url">Twitter settings</a>.', array( '!url' => url('admin/settings/twitter'),
    )));
  }
}

/**
 * Form submit handler for adding a Twiter account.
 *
 * Loads Twitter account details and adds them to the user account
 */
function twitter_auth_account_form_submit($form, &$form_state) {
  $key = variable_get('twitter_consumer_key', '');
  $secret = variable_get('twitter_consumer_secret', '');
  $twitter = new Twitter($key, $secret);
  $token = $twitter->get_request_token();
  if ($token) {
    $_SESSION['twitter_oauth']['token'] = $token;
    $_SESSION['twitter_oauth']['destination'] = $_GET['q'];
    drupal_goto($twitter->get_authorize_url($token));
  }
  else {
    drupal_set_message(t('Could not obtain a valid token from the Twitter API. Please review the configuration.'),
      'error');
  }
}

/**
 * Callback to process the response from the Twitter authentication.
 */
function twitter_oauth_callback(&$form_state) {
  if (isset($_GET['denied']) || empty($_GET['oauth_token'])) {
    drupal_set_message(t('The connection to Twitter failed. Please try again.'), 'error');
    global $user;
    if ($user->uid) {
      // User is logged in, was attempting to OAuth a Twitter account.
      drupal_goto('admin/settings/twitter');
    }
    else {
      // Anonymous user, redirect to front page.
      drupal_goto('<front>');
    }
  }

  $form['#post']['oauth_token'] = $_GET['oauth_token'];
  $form['oauth_token'] = array(
    '#type' => 'hidden',
    '#default_value' => $_GET['oauth_token'],
  );

  return $form;
}

/**
 * Validate results from Twitter OAuth return request.
 */
function twitter_oauth_callback_validate($form, &$form_state) {
  $key = variable_get('twitter_consumer_key', '');
  $secret = variable_get('twitter_consumer_secret', '');

  if (isset($_SESSION['twitter_oauth'])) {
    $form_state['twitter_oauth'] = $_SESSION['twitter_oauth'];
    unset($_SESSION['twitter_oauth']);
  }
  else {
    form_set_error('oauth_token', 'Invalid Twitter OAuth request');
  }

  if (isset($form_state['twitter_oauth']['token'])) {
    $token = $form_state['twitter_oauth']['token'];
    if (!is_array($token) || !$key || !$secret) {
      form_set_error('oauth_token', t('Invalid Twitter OAuth request'));
    }
    if ($token['oauth_token'] != $form_state['values']['oauth_token']) {
      form_set_error('oauth_token', t('Invalid OAuth token.'));
    }
  }
  else {
    form_set_error('oauth_token', t('Invalid Twitter OAuth request'));
  }

  module_load_include('inc', 'twitter');

  if ($twitter = new Twitter($key, $secret, $token['oauth_token'], $token['oauth_token_secret'])) {
    if ($response = $twitter->get_access_token()) {
      $form_state['twitter_oauth']['response'] = $response;
    }
    else {
      form_set_error('oauth_token', t('Invalid Twitter OAuth request'));
    }
  }
  else {
    form_set_error('oauth_token', t('Invalid Twitter OAuth request'));
  }
}

/**
 * Handle a Twitter OAuth return request and store the account creds
 * in the DB.
 */
function twitter_oauth_callback_submit(&$form, &$form_state) {
  $key = variable_get('twitter_consumer_key', '');
  $secret = variable_get('twitter_consumer_secret', '');
  $response = $form_state['twitter_oauth']['response'];

  $twitter = new Twitter($key, $secret, $response['oauth_token'], $response['oauth_token_secret']);
  try {
    $twitter_account = $twitter->users_show($response['screen_name']);
    // Save the new Twitter account and set the user's uid who added it.
    $twitter_account->set_auth($response);
    global $user;
    $twitter_account->added_by_uid = $user->uid;
    twitter_account_save($twitter_account, TRUE);
    drupal_set_message(t('Twitter account added successfully'));
  } catch (TwitterException $e) {
    form_set_error('screen_name', t('Request failed: @message.', array('@message' => $e->getMessage())));
    return;
  }

  $form['#programmed'] = FALSE;
  $form_state['redirect'] = $form_state['twitter_oauth']['destination'];
}

/**
 * Form to add a non-authenticated Twitter account.
 */
function twitter_non_auth_account_form(&$form_state) {
  $form['screen_name'] = array(
    '#type' => 'textfield',
    '#required' => TRUE,
    '#title' => t('Twitter account name'),
    '#prefix' => t('If you simply want to pull tweets from additional Twitter accounts, ' .
                   'enter the Twitter account name below and click on the following button.</br>'),
  );

  $form['submit_non_auth'] = array(
    '#type' => 'submit',
    '#value' => t('Add a non-authenticated account'),
  );

  return $form;
}

/**
 * Submit form handler to add a non-authenticated Twitter account.
 */
function twitter_non_auth_account_form_submit($form, &$form_state) {
  $name = $form_state['values']['screen_name'];
  $twitter = twitter_connect();
  $twitter_account = $twitter->users_show($name, FALSE);
  if (!isset($twitter_account->id)) {
    form_set_error('screen_name', t('Could not add the Twitter account <em>@name</em>. ' .
      'Check the recent messages log.', array('@name' => $name)));
  }
  else {
    global $user;
    $twitter_account->added_by_uid = $user->uid;
    twitter_account_save($twitter_account, FALSE);
    drupal_set_message(t('Twitter account added successfully'));
  }
}
